@@ -25,7 +25,7 @@
 // and as implemented in the kernel module of the Digraphs package for GAP in
 // C:
 //
-// https://github.com/gap-packages/Digraphs/blob/master/src/schreier-sims.c
+// https://github.com/digraphs/Digraphs/blob/master/src/schreier-sims.c
 //
 // by Wilf A. Wilson.
 
@@ -49,24 +49,24 @@
 
 // THIS FILE CAN BE USED AS HEADER ONLY
 
-#ifndef LIBSEMIGROUPS_INCLUDE_SCHREIER_SIMS_HPP_
-#define LIBSEMIGROUPS_INCLUDE_SCHREIER_SIMS_HPP_
+#ifndef LIBSEMIGROUPS_SCHREIER_SIMS_HPP_
+#define LIBSEMIGROUPS_SCHREIER_SIMS_HPP_
 
 #include <array>          // for array
 #include <cstddef>        // for size_t
 #include <string>         // for operator+, basic_string
 #include <unordered_set>  // for unordered_set
 
-#include "adapters.hpp"                 // for action, degree, inverse
-#include "bruidhinn-traits.hpp"         // for detail::BruidhinnTraits
-#include "containers.hpp"               // for Array2, StaticVector2
-#include "int-range.hpp"                // for IntegralRange
-#include "libsemigroups-config.hpp"     // for LIBSEMIGROUPS_HPCOMBI
-#include "libsemigroups-debug.hpp"      // for LIBSEMIGROUPS_ASSERT
-#include "libsemigroups-exception.hpp"  // for LIBSEMIGROUPS_EXCEPTION
-#include "stl.hpp"                      // for EqualTo
-#include "transf.hpp"                   // for Perm
-#include "types.hpp"                    // for SmallestInteger
+#include "adapters.hpp"          // for action, degree, inverse
+#include "bruidhinn-traits.hpp"  // for detail::BruidhinnTraits
+#include "config.hpp"            // for LIBSEMIGROUPS_HPCOMBI_ENABLED
+#include "containers.hpp"        // for Array2, StaticVector2
+#include "debug.hpp"             // for LIBSEMIGROUPS_ASSERT
+#include "exception.hpp"         // for LIBSEMIGROUPS_EXCEPTION
+#include "int-range.hpp"         // for IntegralRange
+#include "stl.hpp"               // for EqualTo
+#include "transf.hpp"            // for Perm
+#include "types.hpp"             // for SmallestInteger
 
 namespace libsemigroups {
 
@@ -82,21 +82,22 @@ namespace libsemigroups {
   //! permutation group to be represented by a SchreierSims instance.
   //! \tparam TPointType the type of the points acted on.
   //! \tparam TElementType the type of the group elements acting on
-  //! ``TPointType``
+  //! \c TPointType.
   template <size_t N, typename TPointType, typename TElementType>
   struct SchreierSimsTraits {
     //! The type of indices to be used by a SchreierSims instance.
     using index_type = size_t;
 
-    //! The type of the object containing all points acted on by a SchreierSims
-    //! instance.
+    //! Type of the object containing all points acted on.
     using domain_type = IntegralRange<TPointType, 0, N>;
 
+    //! Type of the points acted on.
+    //!
     //! The type of the points acted on by the group represented by \c this,
-    //! which is the same as the template parameter ``TPointType``.
+    //! which is the same as the template parameter \c TPointType.
     using point_type = TPointType;
 
-    //! The type of the group elements acting on ``TPointType``
+    //! Type of the elements.
     using element_type = TElementType;
 
     //! \copydoc libsemigroups::ImageRightAction
@@ -129,11 +130,12 @@ namespace libsemigroups {
   //! \tparam N the largest point not fixed by the permutations in the
   //! permutation group to be represented by this.
   //! \tparam TPointType the type of the points acted on (default:
-  //! SmallestInteger<N>::type).
+  //! the member \c type of SmallestInteger with template parameter \p N).
   //! \tparam TElementType the type of the group elements acting on
-  //! ``TPointType`` (default: NewPermHelper<N>::type)
-  //! \tparam TTraits the type of traits object (default: SchreierSimsTraits<N,
-  //! TPointType, TElementType>)
+  //! \c TPointType (default: the member \c type of \ref LeastPerm with template
+  //! parameter \p N).
+  //! \tparam TTraits the type of traits object (default: SchreierSimsTraits
+  //! with template parameters \c N, \c TPointType, and \c TElementType).
   //!
   //! \sa SchreierSimsTraits.
   //!
@@ -147,7 +149,7 @@ namespace libsemigroups {
   //!  \endcode
   template <size_t N,
             typename TPointType   = typename SmallestInteger<N>::type,
-            typename TElementType = typename NewPermHelper<N>::type,
+            typename TElementType = LeastPerm<N>,
             typename TTraits = SchreierSimsTraits<N, TPointType, TElementType>>
   class SchreierSims final : private detail::BruidhinnTraits<TElementType> {
     static_assert(
@@ -167,26 +169,30 @@ namespace libsemigroups {
         TElementType>::internal_const_value_type;
 
    public:
+    //! Type of the elements.
+    //!
     //! The type of the elements of a SchreierSims instance with const removed,
-    //! and if TElementType is a pointer to const, then the second const is
+    //! and if \c TElementType is a pointer to const, then the second const is
     //! also removed.
     using element_type =
         typename detail::BruidhinnTraits<TElementType>::value_type;
 
-    //! The type of the points acted on by the group represented by \c this,
-    //! which is the same as the template parameter TPointType.
+    //! Type of the points acted on.
+    //!
+    //! Also the template parameter \p TPointType.
     using point_type = TPointType;
 
-    //! The type of the object containing all points acted on by a SchreierSims
-    //! instance.
+    //! Type of the object containing all points acted on.
     using domain_type = typename TTraits::domain_type;
 
-    //! The type of indices to be used by a SchreierSims instance.
+    //! Type of indices.
     using index_type = typename TTraits::index_type;
 
-    // gcc apparently requires the extra qualification on the aliases below
-    //! \copydoc libsemigroups::Action
+    //! Alias for \p TTraits::Action.
+    //!
+    //! See Action for further details.
     using Action = typename TTraits::Action;
+
     //! \copydoc libsemigroups::Degree
     using Degree = typename TTraits::Degree;
     //! \copydoc libsemigroups::EqualTo
@@ -210,13 +216,18 @@ namespace libsemigroups {
     };
 
    public:
+    //! Default constructor.
+    //!
     //! Construct a SchreierSims object representing the trivial group.
     //!
     //! \complexity
     //! \f$O(N ^ 2)\f$ where \p N is the first template parameter.
     //!
-    //! \par Parameters
+    //! \parameters
     //! (None)
+    //!
+    //! \exceptions
+    //! \no_libsemigroups_except
     SchreierSims()
         : _base(),
           _base_size(0),
@@ -252,7 +263,7 @@ namespace libsemigroups {
     //! Deleted.
     SchreierSims& operator=(SchreierSims&&) = delete;
 
-    //! Add a generator to this.
+    //! Add a generator.
     //!
     //! \param x a const reference to the generator to add.
     //!
@@ -285,7 +296,7 @@ namespace libsemigroups {
     //!
     //! \complexity
     //! Constant.
-    const_element_reference generator(index_type const index) const {
+    const_element_reference generator(index_type index) const {
       return strong_generator(0, index);
     }
 
@@ -300,10 +311,10 @@ namespace libsemigroups {
     //! \complexity
     //! Constant.
     //!
-    //! \par Parameters
+    //! \parameters
     //! (None)
-    size_t nr_generators() const noexcept {
-      return nr_strong_generators(0);
+    size_t number_of_generators() const noexcept {
+      return number_of_strong_generators(0);
     }
 
     //! The number of strong generators at a given depth.
@@ -320,10 +331,10 @@ namespace libsemigroups {
     //! \complexity
     //! Constant.
     //!
-    //! \par Parameters
+    //! \parameters
     //! (None)
     // TODO(later) shouldn't this throw if depth is out of bounds??
-    size_t nr_strong_generators(index_type const depth) const noexcept {
+    size_t number_of_strong_generators(index_type depth) const noexcept {
       return _strong_gens.size(depth);
     }
 
@@ -341,8 +352,8 @@ namespace libsemigroups {
     //! \complexity
     //! Constant.
     // TODO(later) shouldn't this throw if depth is out of bounds??
-    const_element_reference strong_generator(index_type const depth,
-                                             index_type const index) const {
+    const_element_reference strong_generator(index_type depth,
+                                             index_type index) const {
       if (index >= _strong_gens.size(depth)) {
         LIBSEMIGROUPS_EXCEPTION(
             "index out of bounds, expected value in range (0, %d], got %d",
@@ -354,13 +365,17 @@ namespace libsemigroups {
 
     //! Check if any generators have been added so far.
     //!
-    //! \returns \c true if `nr_generators() == 0` and \c false otherwise.
+    //! \returns \c true if `number_of_generators() == 0` and \c false
+    //! otherwise.
     //!
-    //! \par Parameters
+    //! \parameters
     //! (None)
     //!
     //! \complexity
     //! Constant.
+    //!
+    //! \exceptions
+    //! \no_libsemigroups_except
     // Not noexcept because StaticVector2::size isn't
     bool empty() {
       return _strong_gens.size(0) == 0;
@@ -370,8 +385,11 @@ namespace libsemigroups {
     //!
     //! \returns the size, a value of \c uint64_t.
     //!
-    //! \par Parameters
+    //! \parameters
     //! (None)
+    //!
+    //! \exceptions
+    //! \no_libsemigroups_except
     uint64_t size() {
       if (empty()) {
         return 1;
@@ -384,12 +402,11 @@ namespace libsemigroups {
       return out;
     }
 
-    //! Returns a copy of \p x that has been sifted through the stabiliser
-    //! chain.
+    //! Sift an element through the stabiliser chain.
     //!
     //! \param x a const reference to a group element.
     //!
-    //! \returns A value of type SchreierSims::element_type.
+    //! \returns A value of type \ref element_type.
     //!
     //! \throws LibsemigroupsException if the degree of \p x is not equal to
     //! the first template parameter \c N.
@@ -405,11 +422,14 @@ namespace libsemigroups {
       return cpy;
     }
 
-    //! Returns \c true if \p x belongs to the group.
+    //! Test membership of an element.
     //!
-    //! \param x a const reference to a group element.
+    //! \param x a const reference to the possible element.
     //!
-    //! \returns A bool.
+    //! \returns A \c bool.
+    //!
+    //! \exceptions
+    //! \no_libsemigroups_except
     //!
     //! \note
     //! Returns \c false if the degree of \p x is not equal to the first
@@ -426,13 +446,15 @@ namespace libsemigroups {
       return InternalEqualTo()(_tmp_element2, _one);
     }
 
-    //! Returns a const reference to the identity element of the group
-    //! represented by \c this.
+    //! Returns a const reference to the identity.
     //!
-    //! \returns A bool.
+    //! \returns A \c bool.
     //!
-    //! \par Parameters
+    //! \parameters
     //! (None)
+    //!
+    //! \exceptions
+    //! \no_libsemigroups_except
     const_element_reference identity() const {
       return this->to_external_const(_one);
     }
@@ -448,8 +470,11 @@ namespace libsemigroups {
     //! \complexity
     //! \f$O(N ^ 2)\f$ where \p N is the first template parameter.
     //!
-    //! \par Parameters
+    //! \parameters
     //! (None)
+    //!
+    //! \exceptions
+    //! \no_libsemigroups_except
     void clear() {
       for (size_t depth = 0; depth < N; ++depth) {
         for (size_t index = 0; index < N; ++index) {
@@ -482,8 +507,11 @@ namespace libsemigroups {
     //! \complexity
     //! Constant.
     //!
-    //! \par Parameters
+    //! \parameters
     //! (None)
+    //!
+    //! \exceptions
+    //! \no_libsemigroups_except
     bool finished() const {
       return _finished;
     }
@@ -501,7 +529,7 @@ namespace libsemigroups {
     //!
     //! \complexity
     //! Linear in the current number of base points.
-    void add_base_point(point_type const pt) {
+    void add_base_point(point_type pt) {
       if (pt >= N) {
         LIBSEMIGROUPS_EXCEPTION(
             "base point out of range, expected value in [0, %d), got %d",
@@ -530,7 +558,7 @@ namespace libsemigroups {
     //!
     //! \complexity
     //! Constant.
-    point_type base(index_type const index) const {
+    point_type base(index_type index) const {
       if (index >= _base_size) {
         LIBSEMIGROUPS_EXCEPTION(
             "index out of bounds, expected value in range (0, %d], got %d",
@@ -551,7 +579,7 @@ namespace libsemigroups {
     //! \complexity
     //! Constant.
     //!
-    //! \par Parameters
+    //! \parameters
     //! (None)
     size_t base_size() const noexcept {
       return _base_size;
@@ -566,13 +594,13 @@ namespace libsemigroups {
 
     bool has_valid_degree(const_element_reference x) const {
       return
-#ifdef LIBSEMIGROUPS_HPCOMBI
+#ifdef LIBSEMIGROUPS_HPCOMBI_ENABLED
           std::is_same<HPCombi::Perm16, element_type>::value ||
 #endif
           Degree()(x) == N;
     }
 
-    void internal_add_base_point(point_type const pt) {
+    void internal_add_base_point(point_type pt) {
       LIBSEMIGROUPS_ASSERT(_base_size < N);
       _base[_base_size] = pt;
       _orbits.push_back(_base_size, pt);
@@ -582,7 +610,7 @@ namespace libsemigroups {
       _base_size++;
     }
 
-    void orbit_enumerate(index_type const depth, index_type first = 0) {
+    void orbit_enumerate(index_type depth, index_type first = 0) {
       LIBSEMIGROUPS_ASSERT(depth < _base_size);
       for (index_type i = first; i < _orbits.size(depth); i++) {
         for (auto it = _strong_gens.cbegin(depth);
@@ -593,7 +621,7 @@ namespace libsemigroups {
       }
     }
 
-    void orbit_add_gen(index_type const depth, internal_element_type gen) {
+    void orbit_add_gen(index_type depth, internal_element_type gen) {
       LIBSEMIGROUPS_ASSERT(depth < _base_size);
       // Apply the new generator to existing points in orbits[depth].
       index_type old_size_orbit = _orbits.size(depth);
@@ -603,9 +631,9 @@ namespace libsemigroups {
       orbit_enumerate(depth, old_size_orbit);
     }
 
-    void orbit_add_point(index_type const      depth,
+    void orbit_add_point(index_type            depth,
                          internal_element_type x,
-                         point_type const      pt) {
+                         point_type            pt) {
       point_type img = Action()(pt, this->to_external_const(x));
       if (!_orbits_lookup[depth][img]) {
         _orbits.push_back(depth, img);
@@ -749,4 +777,4 @@ namespace libsemigroups {
   };
 }  // namespace libsemigroups
 
-#endif  // LIBSEMIGROUPS_INCLUDE_SCHREIER_SIMS_HPP_
+#endif  // LIBSEMIGROUPS_SCHREIER_SIMS_HPP_
